@@ -1,7 +1,7 @@
 from mysql.toolkit.components.connector import Connector
-from mysql.toolkit.components.query import Query
+from mysql.toolkit.components.manipulate import Manipulate
 from mysql.toolkit.components.structure import Structure
 from mysql.toolkit.components.operations import Operations
 
 
-__all__ = ['Connector', 'Query', 'Structure', 'Operations']
+__all__ = ['Connector', 'Manipulate', 'Structure', 'Operations']
